import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:our_wedding_sdk/src/model/honey_moon_gift.dart';
import 'package:our_wedding_sdk/src/utils/utils_http.dart';

class HoneyMoonGiftHttp extends UtilsHttp {
  static final HoneyMoonGiftHttp _inviteHttp =
      new HoneyMoonGiftHttp._internal();

  factory HoneyMoonGiftHttp() {
    return _inviteHttp;
  }

  HoneyMoonGiftHttp._internal();

  Future<HoneyMoonGift> postGift({HoneyMoonGift gift, String token}) async {
    final response = await http.post(UtilsHttp.serverIp + 'honeymoongift/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(gift),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 201 && jsonVar != null) {
        HoneyMoonGift ret = HoneyMoonGift.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      return null;
    }
  }

  Future<HoneyMoonGift> putGift({HoneyMoonGift gift, String token}) async {
    HoneyMoonGift ret = HoneyMoonGift();
    var idHoneyMoonGift = gift.idHoneyMoonGift;
    final response = await http
        .put(UtilsHttp.serverIp + 'honeymoongift/$idHoneyMoonGift',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(gift),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        ret = HoneyMoonGift.fromJson(jsonVar);
      }
    } catch (e) {
      ret = null;
    }
    return ret;
  }

  Future<HoneyMoonGift> getGift({int idHoneyMoonGift, String token}) async{
    HoneyMoonGift ret;
    final response = await http
        .get(UtilsHttp.serverIp + 'honeymoongift/$idHoneyMoonGift',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        ret = HoneyMoonGift.fromJson(jsonVar);
      }
    } catch (e) {
      ret = null;
    }
    return ret;
  }

  deleteGift({int idHoneyMoonGift, String token}) async {
    final response = await http
        .delete(UtilsHttp.serverIp + 'honeymoongift/$idHoneyMoonGift',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    return response.statusCode;
  }

  Future<List<HoneyMoonGift>> getAll({String token}) async {
    List<HoneyMoonGift> gifts = [];
    final response = await http
        .get(UtilsHttp.serverIp + 'honeymoongift/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      List value = jsonVar['value'];
      gifts = value.map((giftJson) => HoneyMoonGift.fromJson(giftJson)).toList();
    } catch (e) {
      return gifts;
    }
    gifts.sort((gift1, gift2) => gift1.description.compareTo(gift2.description));
    return gifts;
  }
}
