import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:our_wedding_sdk/src/model/quota.dart';
import 'package:our_wedding_sdk/src/model/quota_totals.dart';
import 'package:our_wedding_sdk/src/utils/utils_http.dart';

class QuotaHttp extends UtilsHttp{
  static final QuotaHttp _quotaHttp = QuotaHttp._internal();

  factory QuotaHttp() {
    return _quotaHttp;
  }

  QuotaHttp._internal();

  Future<Quota> postQuota({@required quota, @required token}) async {
    final response = await http
        .post(UtilsHttp.serverIp + 'quotauser/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(quota),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        Quota ret = Quota.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      print(e.toString());
      return null;
    }
  }

  deleteQuota({int idHoneyMoonGift, int idUser, String token}) async {
    final response = await http
        .delete(UtilsHttp.serverIp + 'quotauser/?idGift=$idHoneyMoonGift&idUser=$idUser',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    return response.statusCode;
  }

  Future<QuotaTotals> calcTotals({int idGift, String token}) async {
    final response = await http
        .get(UtilsHttp.serverIp + 'quotauser/totals/$idGift',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        QuotaTotals ret = QuotaTotals.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      print(e.toString());
      return null;
    }
  }

}