import 'dart:convert';

import 'package:flutter/material.dart';

import 'package:http/http.dart' as http;
import '../../our_wedding_sdk.dart';

class StoreHttp extends UtilsHttp {
  static final StoreHttp _storeHttp = StoreHttp._internal();

  factory StoreHttp() {
    return _storeHttp;
  }

  StoreHttp._internal();

  Future<Store> postStore({@required store, @required token}) async {
    final response = await http.post(UtilsHttp.serverIp + 'store/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(store),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 201 && jsonVar != null) {
        Store ret = Store.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      print(e.toString());
      return null;
    }
  }

  Future<Store> getStore(
      {@required int idStore, @required String token}) async {
    final response = await http.get(UtilsHttp.serverIp + 'store/$idStore',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        Store ret = Store.fromJson(jsonVar);
        return ret;
      }
    } catch (e) {
      return null;
    }
  }

  Future<Store> putStore({Store store, String token}) async {
    Store ret = Store();
    var idStore = store.idStore;
    final response = await http.put(UtilsHttp.serverIp + 'store/$idStore',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        },
        body: json.encode(store),
        encoding: Encoding.getByName("utf-8"));
    try {
      var jsonVar = json.decode(response.body);
      if (response.statusCode == 200 && jsonVar != null) {
        ret = Store.fromJson(jsonVar);
      }
    } catch (e) {}
    return ret;
  }

  deleteStore({int idStore, String token}) async {
    final response = await http.delete(UtilsHttp.serverIp + 'store/$idStore',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    return response.statusCode;
  }

  Future<List<Store>> getAllStores({String token}) async {
    List<Store> stores = [];
    final response = await http
        .get(UtilsHttp.serverIp + 'store/',
        headers: {
          "Content-Type": "application/json",
          "authorization": "Basic " + token
        });
    try {
      var jsonVar = json.decode(response.body);
      List value = jsonVar['value'];
      stores = value.map((userJson) => Store.fromJson(userJson)).toList();

    } catch (e) {
      return stores;
    }
    stores.sort((store1, store2) => store1.name.compareTo(store2.name));
    return stores;
  }
}
