import 'package:flutter_test/flutter_test.dart';
import 'package:our_wedding_sdk/our_wedding_sdk.dart';
import 'package:our_wedding_sdk/src/model/honey_moon_gift.dart';
import 'package:our_wedding_sdk/src/model/invite.dart';
import 'package:our_wedding_sdk/src/model/quota.dart';
import 'package:our_wedding_sdk/src/model/quota_totals.dart';

import 'package:our_wedding_sdk/src/model/user.dart';
import 'package:our_wedding_sdk/src/utils/honey_moon_gift_http.dart';
import 'package:our_wedding_sdk/src/utils/invite_http.dart';
import 'package:our_wedding_sdk/src/utils/quota_http.dart';
import 'package:our_wedding_sdk/src/utils/user_http.dart';
import 'package:our_wedding_sdk/src/utils/utils_http.dart';

void main() {
  group('Create, read, edit, and delete admin user', () {
    _testUserHttp();
  });
  group('Create Invite and invite User', () {
    _testInviteHttp();
  });
  group('Login user', () {
    _testLoginUser();
  });
  group('Honey Moon Gift', (){
    _testHoneyMoonGift();
  });
  group('Read all Honey Moon Gifts', (){
    _testReadAllHoneyMoonGifts();
  });
  group('Quota Tests', (){
    _testQuotas();
  });
  group('Store Tests', (){
    _testStores();
  });
}

void _testStores() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  var store = Store();
  var store1 = Store();
  test('Create Store', () async {
    store = Store(
      imageUrl: 'image.url',
      phone: '1929391',
      link: 'link.com',
      country: 'BR',
      city: 'Curitiba',
      streetAddress: 'Street Address',
      name: 'name'
    );
    store = await StoreHttp().postStore(store: store, token: 'test');
    expect(store.imageUrl, 'image.url');
  });
  test('Create Store1', () async{
    store1 = Store(
        imageUrl: 'image1.url',
        phone: '1929391',
        link: 'link.com',
        country: 'BR',
        city: 'Curitiba',
        streetAddress: 'Street Address',
        name: 'name'
    );
    store1 = await StoreHttp().postStore(store: store1, token: 'test');
    expect(store1.imageUrl, 'image1.url');
  });
  test('Test get Store', () async {
   var _store1 = await StoreHttp().getStore(idStore: store.idStore, token: 'test');
   expect(_store1.idStore, store.idStore);
  });
  test('Test PUT Store', () async {
    store.country = 'DE';
    var _store1 = await StoreHttp().putStore(store: store, token: 'test');
    expect(_store1.country, store.country);
  });
  test('GET all stores', () async{
    List<Store> _stores = await StoreHttp().getAllStores(token: 'test');
    List<Store> _stores1 = [];
    _stores.forEach((_sto){
      if(_sto.idStore == store.idStore || _sto.idStore == store1.idStore){
        _stores1.add(_sto);
      }
    });
    expect(_stores1.length, 2);
  });
  test('Test DELETE Store', () async{
    var responseCode = await StoreHttp().deleteStore(idStore: store.idStore, token: 'test');
    expect(responseCode, 204);
  });
  test('Test DELETE Store1', () async{
    var responseCode = await StoreHttp().deleteStore(idStore: store1.idStore, token: 'test');
    expect(responseCode, 204);
  });
}

void _testQuotas() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  var user1 = User();
  var user2 = User();
  var gift1 = HoneyMoonGift();
  var quota1 = Quota();
  var quota2 = Quota();
  test('Create 1st User', () async{
    user1 = User(
        email: 'flutter4@gmail.com',
        userName: 'Flutter User',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 1,
        confirmed: 0);
    user1 = await UserHttp().postUser(user: user1, token: 'test');
    expect(user1.userName, 'Flutter User');
    expect(user1.confirmed, 0);
  });
  test('Create 2nd User', () async{
    user2 = User(
        email: 'flutter5@gmail.com',
        userName: 'Flutter User Name',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 1,
        confirmed: 0);
    user2 = await UserHttp().postUser(user: user2, token: 'test');
    expect(user2.userName, 'Flutter User Name');
    expect(user2.confirmed, 0);
  });
  test('Create 1st. Honey Moon Gift', () async{
    gift1 = HoneyMoonGift(
        description: 'Description 1st',
        totalValue: 25.25,
        imageUrl: 'image.com.br',
        quota: 5,
        city: 'Curitiba',
        country: 'BR',
        internetAddress: 'anyaddress.com.br',
        streetAddress: 'Any Street 14'
    );
    gift1 = await HoneyMoonGiftHttp().postGift(gift: gift1, token: 'test');
    expect(gift1.description, 'Description 1st');
  });
  test('Create 1st quota', () async{
    quota1 = Quota(
      date: DateTime.now(),
      vlrQuota: 50.0,
      qtdQuota: 2,
      idUser: user1.idUser,
      idHoneyMoonGift: gift1.idHoneyMoonGift
    );
    var newQuota = await QuotaHttp().postQuota(quota: quota1, token: 'test');
    expect(quota1.idHoneyMoonGift, newQuota.idHoneyMoonGift);
  });
  test('Create 2nd quota', () async{
    quota2 = Quota(
        date: DateTime.now(),
        vlrQuota: 25.0,
        qtdQuota: 1,
        idUser: user2.idUser,
        idHoneyMoonGift: gift1.idHoneyMoonGift
    );
    var newQuota = await QuotaHttp().postQuota(quota: quota2, token: 'test');
    expect(quota2.idHoneyMoonGift, newQuota.idHoneyMoonGift);
  });
  test('Calc totals quota', () async{
    QuotaTotals totals = await QuotaHttp().calcTotals(idGift: gift1.idHoneyMoonGift,
      token: 'test');
    expect(totals.vlrTotals, 75.0);
    expect(totals.qtdTotals, 3);
  });
  test('Delete 1st quota', () async{
    var responseCode = await QuotaHttp().deleteQuota(idHoneyMoonGift: quota1.idHoneyMoonGift,
        idUser: quota1.idUser, token: 'test');
    expect(responseCode, 204);
  });
  test('Delete 2nd quota', () async{
    var responseCode = await QuotaHttp().deleteQuota(idHoneyMoonGift: quota2.idHoneyMoonGift,
        idUser: quota2.idUser, token: 'test');
    expect(responseCode, 204);
  });
  test('Delete 1st Honey Moon Gift', () async{
    var responseCode = await HoneyMoonGiftHttp().deleteGift(idHoneyMoonGift: gift1.idHoneyMoonGift,
        token: 'test');
    expect(responseCode, 204);
  });
  test('Delete 1st User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: user1.idUser,
        adminUser: user1.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
  test('Delete 2nd User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: user2.idUser,
        adminUser: user2.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
}

void _testReadAllHoneyMoonGifts() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  var gift1 = HoneyMoonGift();
  var gift2 = HoneyMoonGift();
  var gift3 = HoneyMoonGift();
  test('Create 1st. Honey Moon Gift', () async{
    gift1 = HoneyMoonGift(
        description: 'Description 1st',
        totalValue: 25.25,
        imageUrl: 'image.com.br',
        quota: 5,
        city: 'Curitiba',
        country: 'BR',
        internetAddress: 'anyaddress.com.br',
        streetAddress: 'Any Street 14'
    );
    gift1 = await HoneyMoonGiftHttp().postGift(gift: gift1, token: 'test');
    expect(gift1.country, 'BR');
  });
  test('Create 2nd. Honey Moon Gift', () async{
    gift2 = HoneyMoonGift(
        description: 'Description 2nd',
        totalValue: 25.25,
        imageUrl: 'image.com.br',
        quota: 5,
        city: 'Curitiba',
        country: 'BR',
        internetAddress: 'anyaddress.com.br',
        streetAddress: 'Any Street 14'
    );
    gift2 = await HoneyMoonGiftHttp().postGift(gift: gift2, token: 'test');
    expect(gift2.description, 'Description 2nd');
  });
  test('Create 3rd. Honey Moon Gift', () async{
    gift3 = HoneyMoonGift(
        description: 'Description 3rd',
        totalValue: 25.25,
        imageUrl: 'image.com.br',
        quota: 5,
        city: 'Curitiba',
        country: 'BR',
        internetAddress: 'anyaddress.com.br',
        streetAddress: 'Any Street 14'
    );
    gift3 = await HoneyMoonGiftHttp().postGift(gift: gift3, token: 'test');
    expect(gift3.description, 'Description 3rd');
  });
  test('Read all gifts', () async{
    List<HoneyMoonGift> list = await HoneyMoonGiftHttp().getAll(token: 'test');
    List<HoneyMoonGift> list1 = [];
    list.forEach((gft){
      if(gft.idHoneyMoonGift == gift1.idHoneyMoonGift ||
        gft.idHoneyMoonGift == gift2.idHoneyMoonGift ||
        gft.idHoneyMoonGift == gift3.idHoneyMoonGift){
        list1.add(gft);
      }
    });
    expect(list1.length, 3);
  });
  test('Delete 1st Honey Moon Gift', () async{
    var responseCode = await HoneyMoonGiftHttp().deleteGift(idHoneyMoonGift: gift1.idHoneyMoonGift,
        token: 'test');
    expect(responseCode, 204);
  });
  test('Delete 2nd Honey Moon Gift', () async{
    var responseCode = await HoneyMoonGiftHttp().deleteGift(idHoneyMoonGift: gift2.idHoneyMoonGift,
        token: 'test');
    expect(responseCode, 204);
  });
  test('Delete 3rd Honey Moon Gift', () async{
    var responseCode = await HoneyMoonGiftHttp().deleteGift(idHoneyMoonGift: gift3.idHoneyMoonGift,
        token: 'test');
    expect(responseCode, 204);
  });
}

void _testHoneyMoonGift() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  var gift = HoneyMoonGift();
  test('Create Honey Moon Gift', () async{
    gift = HoneyMoonGift(
      description: 'Description',
      totalValue: 25.25,
      imageUrl: 'image.com.br',
      quota: 5,
      city: 'Curitiba',
      country: 'BR',
      internetAddress: 'anyaddress.com.br',
      streetAddress: 'Any Street 14'
    );
    gift = await HoneyMoonGiftHttp().postGift(gift: gift, token: 'test');
    expect(gift.description, 'Description');
  });
  test('Edit Honey Moon Gift', () async{
    var descr = 'Description 2';
    gift.description = descr;
    gift = await HoneyMoonGiftHttp().putGift(gift: gift, token: 'test');
    expect(gift.description, descr);
  });
  test('Read Honey Moon Gift', () async {
    HoneyMoonGift gift2 = await HoneyMoonGiftHttp().getGift(idHoneyMoonGift: gift.idHoneyMoonGift,
      token: 'test');
    expect(gift2.idHoneyMoonGift, gift.idHoneyMoonGift);
  });
  test('Delete Honey Moon Gift', () async{
    var responseCode = await HoneyMoonGiftHttp().deleteGift(idHoneyMoonGift: gift.idHoneyMoonGift,
      token: 'test');
    expect(responseCode, 204);
  });
}

void _testLoginUser() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  var user = User();
  var loginUser = User();
  test('Create User', () async{
    user = User(
        email: 'flutter4@gmail.com',
        userName: 'Flutter User',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 1,
        confirmed: 0);
    user = await UserHttp().postUser(user: user, token: 'test');
    expect(user.userName, 'Flutter User');
    expect(user.confirmed, 0);
  });
  test('Login User', () async{
    loginUser = await UserHttp().loginUser(email: user.email, token: 'test');
    expect(user.email, loginUser.email);
  });
  test('Delete User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: user.idUser,
        adminUser: user.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
}

void _testInviteHttp() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  int idAdminUser;
  var adminUser = User();
  var guestUser = User();
  var invite = Invite();
  test('Create Admin User', () async {
    adminUser = User(
        email: 'flutter4@gmail.com',
        userName: 'Flutter User',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 1);
    adminUser = await UserHttp().postUser(user: adminUser, token: 'test');
    expect(adminUser.userName, 'Flutter User');
    idAdminUser = adminUser.idUser;
  });
  test('Create Guest User', () async {
    guestUser = User(
        email: 'guest1@gmail.com',
        userName: 'Guest User',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 2);
    guestUser = await UserHttp().postUser(user: guestUser, token: 'test');
    expect(guestUser.userName, 'Guest User');
  });
  test('Create Invite', () async {
    invite =
        Invite(idInvite: 'AAAAAE', date: DateTime.now(), idUser: idAdminUser);
    invite = await InviteHttp().postInvite(invite: invite, token: 'test');
    expect(invite.idInvite, 'AAAAAE');
  });
  test('Read Invite',() async {
    invite = await InviteHttp().getInvite(idInvite: 'AAAAAE', token: 'test');
    expect(invite.idInvite, 'AAAAAE');
  });
  test('Invite User', () async {
    guestUser.idInvite = invite.idInvite;
    guestUser = await UserHttp().putUser(user: guestUser, token: 'test');
    expect(guestUser.idInvite, invite.idInvite);
  });
  test('Load Invite User', () async{
    List<User> users = await UserHttp().getInviteUsers(invite: invite, token: 'test');
    expect(users.length, 1);
  });
  test('Delete Guest User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: guestUser.idUser,
        adminUser: adminUser.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
  test('Delete Invite', () async {
    var responseCode =
        await InviteHttp().deleteInvite(idInvite: 'AAAAAE', token: 'test');
    expect(responseCode, 204);
  });
  test('Delete Admin User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: adminUser.idUser,
        adminUser: adminUser.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
}

void _testUserHttp() {
  UtilsHttp.init('http://192.168.15.10:8080/api/v1/');
  int idUser;
  var adminUser = User();
  var guestUser = User();
  test('Create Admin User', () async {
    var user = User(
        email: 'flutter@gmail.com',
        userName: 'Flutter User',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 1);
    user = await UserHttp().postUser(user: user, token: 'test');
    expect(user.userName, 'Flutter User');
    idUser = user.idUser;
  });
  test('Create Guest User', () async {
    guestUser = User(
        email: 'guest1@gmail.com',
        userName: 'Guest User',
        city: 'Flutter City',
        avatarUrl: 'some.avatar.com',
        profile: 2);
    guestUser = await UserHttp().postUser(user: guestUser, token: 'test');
    expect(guestUser.userName, 'Guest User');
  });
  adminUser = User();
  test('Read User', () async {
    adminUser = await UserHttp().getUser(idUser: idUser, token: 'test');
    expect(adminUser.idUser, idUser);
  });
  test('Update User', () async {
    adminUser.userName = 'New Flutter User';
    adminUser = await UserHttp().putUser(user: adminUser, token: 'test');
    expect(adminUser.userName, 'New Flutter User');
  });
  test('Confirm User', () async{
    guestUser.confirmed = 1;
    guestUser = await UserHttp().putUser(user: guestUser, token: 'test');
    expect(guestUser.confirmed, 1);
  }

  );
  test('Delete with illegal user', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: adminUser.idUser,
        adminUser: guestUser.idUser,
        token: 'test');
    expect(responseCode, 401);
  });
  test('Delete Guest User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: guestUser.idUser,
        adminUser: adminUser.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
  test('Delete User', () async {
    var responseCode = await UserHttp().deleteUser(
        deleteUser: adminUser.idUser,
        adminUser: adminUser.idUser,
        token: 'test');
    expect(responseCode, 204);
  });
}
